Package: DelayedTensor
Type: Package
Title: R package for sparse and out-of-core arithmetic and
        decomposition of Tensor
Version: 1.19.0
Authors@R: c(person("Koki", "Tsuyuzaki", role = c("aut", "cre"),
                email = "k.t.the-answer@hotmail.co.jp"))
Depends: R (>= 4.1.0)
Imports: methods, utils, S4Arrays, SparseArray, DelayedArray (>=
        0.31.8), HDF5Array, BiocSingular, rTensor, DelayedRandomArray
        (>= 1.13.1), irlba, Matrix, einsum,
Suggests: markdown, rmarkdown, BiocStyle, knitr, testthat, magrittr,
        dplyr, reticulate
Description: DelayedTensor operates Tensor arithmetic directly on
    DelayedArray object. DelayedTensor provides some generic function related to
    Tensor arithmetic/decompotision and dispatches it on the DelayedArray class.
    DelayedTensor also suppors Tensor contraction by einsum function,
    which is inspired by numpy einsum.
License: Artistic-2.0
biocViews: Software, Infrastructure, DataRepresentation,
        DimensionReduction
LazyData: true
LazyDataCompression: xz
VignetteBuilder: knitr
BugReports: https://github.com/rikenbit/DelayedTensor/issues
git_url: https://git.bioconductor.org/packages/DelayedTensor
git_branch: devel
git_last_commit: ebd03f4
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-29
NeedsCompilation: no
Packaged: 2026-04-29 21:22:06 UTC; biocbuild
Author: Koki Tsuyuzaki [aut, cre]
Maintainer: Koki Tsuyuzaki <k.t.the-answer@hotmail.co.jp>
