\name{getVerbose}
\alias{getVerbose}
\title{Getter function to control the verbose messages from DelayedTensor}
\usage{
getVerbose()
}
\value{
TRUE or FALSE (Default: FALSE)
}
\description{
Returns the verbose setting of DelayedTensor functions.
}
\examples{
getVerbose()
}
